# Copyright (C) 1999, 2000 Jay Beale
# Licensed under the GNU General Public License

package Bastille::AccountSecurity;


use Bastille::API;

@ENV="";
$ENV{PATH}="";
$ENV{CDPATH}=".";
$ENV{BASH_ENV}="";

# 5/3/2000 -- new version created from old by JJB

#  TO DO:
#        1) Document by copying text from questions file...

#######################################################################
##                     Account Creation/Security                     ##
#######################################################################

&CreateAdminAccount;
&ProtectRhosts;
&PasswordAging;
&CreateUser;       
&RestrictCron;


sub CreateAdminAccount {
	my ($rootHome,@pwInfo);
# We create a UID 0 account.  Don't underestimate this step in 
# tracking the actions of hackers of lesser ability who are able to
# grab root on your box.  By never using the real root account, root
# logins/.../actions become warning signs.


    if ( ( $GLOBAL_CONFIG{"AccountSecurity"}{"secondadmin"} eq "Y" ) and ( $GLOBAL_CONFIG{"AccountSecurity"}{"secondadmin_name"}) ) {

	my $admin_name=$GLOBAL_CONFIG{"AccountSecurity"}{"secondadmin_name"};
	chomp $admin_name;

	&ActionLog("# sub CreateAdminAccount\n");


	# Add the admin account to the passwd file
	&B_append_line("/etc/passwd","^$admin_name:","$admin_name:x:0:0:$admin_name:" . $GLOBAL_DIR{"home"} . "/$admin_name:" . $GLOBAL_BIN{"bash"} . "\n");
	
	# Add the admin account to the shadow file
	&B_append_line("/etc/shadow","^$admin_name:","$admin_name:*:11088:0:99999:7:::\n");

	# Create the admin's home directory
	&B_create_dir("/home/$admin_name");

	# Make sure this new uid 0 account can't use ftp.
	# Redhat defaults to this stance for the root account.

	&B_append_line ("/etc/ftpusers","^\\s*$admin_name\\s*\$","$admin_name\n");

        # Play with the history files on the real root account and the new
        # root account...

	# make sure we have the right home dir for root
	$rootHome = "/root";
	while ( @pwInfo = getpwent()) {
		if ($pwInfo[0] eq "root") { $rootHome = $pwInfo[7]; }
	}
	&B_append_line("${rootHome}/.bashrc",'^\s*HISTFILE',"HISTFILE=\"/.root_hist\"\nexport HISTFILE\n");

        &B_create_file("/home/$admin_name/.bashrc");
        &B_append_line("/home/$admin_name/.bashrc",'^\s*HISTSIZE=',"HISTSIZE=10\n");
        &B_append_line("/home/$admin_name/.bashrc",'^\s*HISTFILESIZE=',"HISTFILESIZE=10\n");
        &B_append_line("/home/$admin_name/.bashrc",'^\s*HISTFILE=',"HISTFILE=/home/$admin_name/.hst\n");
        &B_append_line("/home/$admin_name/.bashrc",'export HISTSIZE HISTFILESIZE',"export HISTSIZE HISTFILESIZE HISTFILE\n");

    }
}    


sub ProtectRhosts {

    if ( $GLOBAL_CONFIG{"AccountSecurity"}{"protectrhost"} eq "Y" ) {

	&ActionLog("# sub ProtectRhosts\n");

	# Disallow rlogin,rsh,rexec via Pluggable Authentication Modules
	foreach $service ( "rexec","rlogin","rsh" ) {
	    &B_prepend_line($GLOBAL_DIR{"pamd"} . "/$service","pam_deny","auth      required   pam_deny.so\n");
	}
	
	# Deactivate the daemons by removing execute status
	&B_chmod(000,"/usr/sbin/in.rlogind");
	&B_chmod(000,"/usr/sbin/in.rshd");
	&B_chmod(000,"/usr/sbin/in.rexecd");
	
	# Comment out rsh/rlogin/rexec lines from inetd.conf
	&B_hash_comment_line("/etc/inetd.conf","rlogind");
	&B_hash_comment_line("/etc/inetd.conf","rshd");
	&B_hash_comment_line("/etc/inetd.conf","rexecd");
	
    }
}


sub PasswordAging {

# Set default password aging, such that accounts are disabled if the
# password is not changed every 180 days.  We use this hopefully to keep
# passwords fresh and automatically disable accounts that haven't been 
# used in a while.  We could create a cron job that parses lastlog looking
# for unused accounts, but that would fail if your lastlog got deleted by
# an attacker or cycled by your log cycler, as the cron job would disable
# many accounts...

    if ($GLOBAL_CONFIG{"AccountSecurity"}{"passwdage"} eq "Y") {

	&ActionLog("# sub Password Aging\n");
	
        my $got_replaced=&B_replace_line("/etc/login.defs","^\s*PASS_MAX_DAYS","PASS_MAX_DAYS   180\n");
        unless ($got_replaced) {
            B_append_line("/etc/login.defs",'^\s*PASS_MAX_DAYS',"PASS_MAX_DAYS   180\n");
        }
   }
					      
}



sub CreateUser {

    if ( ($GLOBAL_CONFIG{"AccountSecurity"}{"createuser"} eq "Y") and ( $GLOBAL_CONFIG{"AccountSecurity"}{"createuser_name"} ) ) {

	&ActionLog("# sub CreateUser\n");

	my $user_name=$GLOBAL_CONFIG{"AccountSecurity"}{"createuser_name"};

	# Grab the next UID/GID pair and create an account...
	my $next_uid=500;
	my $next_gid=500;
	
	if (&B_open(*PASSWD,"$GLOBAL_PREFIX/etc/passwd")) {

	    # Bug fix 1.1.0->1.1.1 : We scan all lines in /etc/passwd to
	    #                        compute UID/GID now...

	    while (my $line=<PASSWD>) {
		my $uid = (split /:/,$line)[2];
		my $gid = (split /:/,$line)[3];
		if ( $uid >= $next_uid ) {
		    $next_uid=$uid+1;
		}
		if ( $gid >= $next_gid ) {
		    $next_gid=$gid+1;
		}
	    }

	    # We've got a uid/gid that is bigger than all the others
	    &B_close(*PASSWD);
	    
	    my $new_line="$user_name:x:$next_uid:$next_gid:$user_name:" . $GLOBAL_DIR{"home"} . "/$user_name:" . $GLOBAL_BIN{"bash"} . "\n";
	    &B_append_line("/etc/passwd","$user_name:",$new_line);
	    &B_append_line("/etc/shadow","$user_name:","$user_name:x:11088:0:99999:7:::\n");	
	    
	    my $home_dir = $GLOBAL_DIR{"home"} . "/$user_name";
	    &B_create_dir($home_dir);
	    &B_chown($next_uid,$home_dir);
	    &B_chgrp($next_gid,$home_dir);
	}
    }
    
}


sub RestrictCron {

   if ( $GLOBAL_CONFIG{"AccountSecurity"}{"cronuser"} eq "Y" ) {

       &ActionLog("# sub RestrictCron\n");

       # Create an /etc/cron.allow file, thus restricting crontab's use to
       # root, the second admin and the user account we created.
       
       my $admin_name=$GLOBAL_CONFIG{"AccountSecurity"}{"secondadmin_name"};
       my $user_name=$GLOBAL_CONFIG{"AccountSecurity"}{"createuser_name"};

       # Create the cron.allow file, unless it already exists.
       &B_create_file ("/etc/cron.allow");

       # Append a list of accounts that can use cron -- only adding lines
       # if they're not yet in there...

       &B_append_line ("/etc/cron.allow","root","root\n");
       
       if ($GLOBAL_CONFIG{"AccountSecurity"}{"secondadmin_name"}) { 
	   &B_append_line ("/etc/cron.allow","$admin_name","$admin_name\n");
       }
       if ($GLOBAL_CONFIG{"AccountSecurity"}{"createuser_name"}) {
	   &B_append_line ("/etc/cron.allow","$user_name","$user_name\n");
       }

   }
}

1;

